@@ -40,7 +40,6 @@
 
 #include <sofia-sip/su_alloc.h>
 #include <sofia-sip/su_string.h>
-#include <sofia-sip/su_bm.h>
 
 #include "msg_internal.h"
 #include "sofia-sip/msg.h"
@@ -56,6 +55,11 @@
 #include <errno.h>
 #include <assert.h>
 
+#if !HAVE_MEMMEM
+void *memmem(const void *haystack, size_t haystacklen,
+	     const void *needle, size_t needlelen);
+#endif
+
 /** Protocol version of MIME */
 char const msg_mime_version_1_0[] = "MIME/1.0";
 
@@ -268,7 +272,7 @@ msg_multipart_boundary(su_home_t *home, char const *b)
   if (!b || !(boundary = su_alloc(home, 2 + 2 + strlen(b) + 2 + 1)))
     return NULL;
 
-  strcpy(boundary, CR LF "--");
+  strcpy(boundary, _CR LF "--");
 
   if (b[0] == '"') /* " See http://bugzilla.gnome.org/show_bug.cgi?id=134216 */
 
@@ -277,7 +281,7 @@ msg_multipart_boundary(su_home_t *home, char const *b)
     strcpy(boundary + 4, b);
 
 
-  strcat(boundary + 4, CR LF);
+  strcat(boundary + 4, _CR LF);
 
   return boundary;
 }
@@ -317,16 +321,16 @@ msg_multipart_search_boundary(su_home_t *home, char const *p, size_t len)
     if (m > 2 && crlf) {
       boundary = su_alloc(home, 2 + m + 2 + 1);
       if (boundary) {
-	memcpy(boundary, CR LF, 2);
+	memcpy(boundary, _CR LF, 2);
 	memcpy(boundary + 2, p, m);
-	strcpy(boundary + m + 2, CR LF);
+	strcpy(boundary + m + 2, _CR LF);
       }
       return boundary;
     }
   }
 
   /* Look for LF -- */
-  for (;(p = bm_memmem(p, end - p, LF "--", 3, NULL)); p += 3) {
+  for (;(p = memmem(p, end - p, LF "--", 3)); p += 3) {
     len = end - p;
     m = 3 + su_memspn(p + 3, len - 3, bchars, bchars_len);
     if (m + 2 >= len)
@@ -338,9 +342,9 @@ msg_multipart_search_boundary(su_home_t *home, char const *p, size_t len)
     if (m > 2 && crlf) {
       boundary = su_alloc(home, 2 + m + 2 + 1);
       if (boundary) {
-	memcpy(boundary, CR LF, 2);
+	memcpy(boundary, _CR LF, 2);
 	memcpy(boundary + 2, p + 1, m);
-	strcpy(boundary + 2 + m, CR LF);
+	strcpy(boundary + 2 + m, _CR LF);
       }
       return boundary;
     }
@@ -374,9 +378,10 @@ msg_multipart_t *msg_multipart_parse(su_home_t *home,
   msg_t msg[1] = {{{ SU_HOME_INIT(msg) }}};
   size_t len, m, blen;
   char *boundary, *p, *next, save;
-  char const *b, *end;
+  char *b, *end;
   msg_param_t param;
-  bm_fwd_table_t *fwd = NULL;
+
+  if (!pl) return NULL;
 
   p = pl->pl_data; len = pl->pl_len; end = p + len;
 
@@ -395,20 +400,16 @@ msg_multipart_t *msg_multipart_parse(su_home_t *home,
 
   m = strlen(boundary) - 2, blen = m - 1;
 
-  if (blen > 8 * sizeof (long))
-    fwd = bm_memmem_study(boundary + 1, blen);
-
   /* Find first delimiter */
   if (memcmp(boundary + 2, p, m - 2) == 0)
     b = p, p = p + m - 2, len -= m - 2;
-  else if ((p = bm_memmem(p, len, boundary + 1, blen, fwd))) {
+  else if ((p = memmem(p, len, boundary + 1, m - 1))) {
     if (p != pl->pl_data && p[-1] == '\r')
       b = --p, p = p + m, len -= m;
     else
       b = p, p = p + m - 1, len -= m - 1;
   }
   else {
-    free(fwd);
     su_home_deinit(msg_home(msg));
     return NULL;
   }
@@ -425,7 +426,7 @@ msg_multipart_t *msg_multipart_parse(su_home_t *home,
     if (len < blen)
       break;
 
-    next = bm_memmem(p, len, boundary + 1, m = blen, fwd);
+    next = memmem(p, len, boundary + 1, m = blen);
 
     if (!next)
       break;			/* error */
@@ -439,8 +440,11 @@ msg_multipart_t *msg_multipart_parse(su_home_t *home,
     *mmp = mp; mmp = &mp->mp_next;
 
     /* Put delimiter transport-padding CRLF here */
-    mp->mp_common->h_data = b;
+
+	*b = '\0';
     mp->mp_common->h_len = p - b;
+	b += strlen(boundary) - 2;
+    mp->mp_common->h_data = b;
 
     /* .. and body-part here */
     mp->mp_data = p;
@@ -450,22 +454,23 @@ msg_multipart_t *msg_multipart_parse(su_home_t *home,
       /* We found close-delimiter */
       assert(mp);
       if (!mp)
-	break;			/* error */
+		  break;			/* error */
       mp->mp_close_delim = (msg_payload_t *)
-	msg_header_alloc(msg_home(msg), msg_payload_class, 0);
+		  msg_header_alloc(msg_home(msg), msg_payload_class, 0);
       if (!mp->mp_close_delim)
-	break;			/* error */
+		  break;			/* error */
       /* Include also transport-padding and epilogue in the close-delimiter */
-      mp->mp_close_delim->pl_data = next;
+	  *next = '\0';
       mp->mp_close_delim->pl_len = p + len - next;
+	  next += strlen(boundary) - 2;
+      mp->mp_close_delim->pl_data = next;
+	  
       break;
     }
 
     b = next; p = next + m;
   }
 
-  free(fwd), fwd = NULL;
-
   if (!mp || !mp->mp_close_delim) {
     su_home_deinit(msg_home(msg));
     /* Delimiter error */
@@ -518,8 +523,8 @@ msg_multipart_t *msg_multipart_parse(su_home_t *home,
     mp->mp_data = boundary;
     mp->mp_len = (unsigned)blen; /* XXX */
 
-    assert(mp->mp_payload || mp->mp_separator);
-
+    if (!(mp->mp_payload || mp->mp_separator)) continue;
+	
     if (mp->mp_close_delim) {
       msg_header_t **tail;
 
@@ -586,7 +591,7 @@ int msg_multipart_complete(su_home_t *home,
       b = mp->mp_data;
       m = mp->mp_len;
 
-      if (strncmp(b, CR LF "--", 4) == 0)
+      if (strncmp(b, _CR LF "--", 4) == 0)
 	b += 4, m -= 4;
       else if (strncmp(b, "--", 2) == 0)
 	b += 2, m -= 2;
@@ -635,7 +640,7 @@ int msg_multipart_complete(su_home_t *home,
 
     if (mp->mp_next == NULL) {
       if (!mp->mp_close_delim)
-	mp->mp_close_delim = msg_payload_format(home, "%.*s--" CR LF,
+	mp->mp_close_delim = msg_payload_format(home, "%.*s--" _CR LF,
 						(int)m, boundary);
       if (!mp->mp_close_delim)
 	return -1;
@@ -655,7 +660,7 @@ int msg_multipart_complete(su_home_t *home,
     mp->mp_common->h_len = mp->mp_len;
 
     if (!mp->mp_separator)
-      if (!(mp->mp_separator = msg_separator_make(home, CR LF)))
+      if (!(mp->mp_separator = msg_separator_make(home, _CR LF)))
 	return -1;
 
     if (mp->mp_multipart) {
@@ -1066,28 +1071,33 @@ MSG_HEADER_CLASS(msg_, accept, "Accept", "", ac_params, apndlist,
 
 issize_t msg_accept_d(su_home_t *home, msg_header_t *h, char *s, isize_t slen)
 {
-  msg_accept_t *ac = (msg_accept_t *)h;
+	msg_accept_t *ac;
 
-  while (*s == ',')   /* Ignore empty entries (comma-whitespace) */
-    *s = '\0', s += span_lws(s + 1) + 1;
+	for(;;) {
+		ac = (msg_accept_t *)h;
 
-  if (*s == '\0') {
-    /* Empty Accept list is not an error */
-    ac->ac_type = ac->ac_subtype = "";
-    return 0;
-  }
+		while (*s == ',')   /* Ignore empty entries (comma-whitespace) */
+			*s = '\0', s += span_lws(s + 1) + 1;
 
-  /* "Accept:" #(type/subtyp ; *(parameters))) */
-  if (msg_mediatype_d(&s, &ac->ac_type) == -1)
-    return -1;
-  if (!(ac->ac_subtype = strchr(ac->ac_type, '/')))
-    return -1;
-  ac->ac_subtype++;
+		if (*s == '\0') {
+			/* Empty Accept list is not an error */
+			ac->ac_type = ac->ac_subtype = "";
+			return 0;
+		}
 
-  if (*s == ';' && msg_params_d(home, &s, &ac->ac_params) == -1)
-    return -1;
+		/* "Accept:" #(type/subtyp ; *(parameters))) */
+		if (msg_mediatype_d(&s, &ac->ac_type) == -1)
+			return -1;
+		if (!(ac->ac_subtype = strchr(ac->ac_type, '/')))
+			return -1;
+		ac->ac_subtype++;
+
+		if (*s == ';' && msg_params_d(home, &s, &ac->ac_params) == -1)
+			return -1;
+  
+		msg_parse_next_field_without_recursion();
+	}
 
-  return msg_parse_next_field(home, h, s, slen);
 }
 
 issize_t msg_accept_e(char b[], isize_t bsiz, msg_header_t const *h, int flags)
@@ -1164,22 +1174,26 @@ issize_t msg_accept_any_d(su_home_t *home,
 			  char *s, isize_t slen)
 {
   /** @relatesalso msg_accept_any_s */
-  msg_accept_any_t *aa = (msg_accept_any_t *)h;
+	msg_accept_any_t *aa;
 
-  while (*s == ',')   /* Ignore empty entries (comma-whitespace) */
-    *s = '\0', s += span_lws(s + 1) + 1;
+	for(;;) {
+		aa = (msg_accept_any_t *)h;
+		while (*s == ',')   /* Ignore empty entries (comma-whitespace) */
+			*s = '\0', s += span_lws(s + 1) + 1;
 
-  if (*s == '\0')
-    return -2;			/* Empty list */
+		if (*s == '\0')
+			return -2;			/* Empty list */
 
-  /* "Accept-*:" 1#(token *(SEMI accept-param)) */
-  if (msg_token_d(&s, &aa->aa_value) == -1)
-    return -1;
+		/* "Accept-*:" 1#(token *(SEMI accept-param)) */
+		if (msg_token_d(&s, &aa->aa_value) == -1)
+			return -1;
 
-  if (*s == ';' && msg_params_d(home, &s, &aa->aa_params) == -1)
-    return -1;
+		if (*s == ';' && msg_params_d(home, &s, &aa->aa_params) == -1)
+			return -1;
+		
+		msg_parse_next_field_without_recursion();
+	}
 
-  return msg_parse_next_field(home, h, s, slen);
 }
 
 /** Encode an Accept-* header field. */
@@ -2068,29 +2082,33 @@ MSG_HEADER_CLASS_G(content_transfer_encoding, "Content-Transfer-Encoding",
 
 issize_t msg_warning_d(su_home_t *home, msg_header_t *h, char *s, isize_t slen)
 {
-  msg_warning_t *w = (msg_warning_t *)h;
+	msg_warning_t *w;
   char *text;
 
-  while (*s == ',')   /* Ignore empty entries (comma-whitespace) */
-    *s = '\0', s += span_lws(s + 1) + 1;
+  for(;;) {
+	  w = (msg_warning_t *)h;
+	  while (*s == ',')   /* Ignore empty entries (comma-whitespace) */
+		  *s = '\0', s += span_lws(s + 1) + 1;
 
-  /* Parse protocol */
-  if (!IS_DIGIT(*s))
-    return -1;
-  w->w_code = strtoul(s, &s, 10);
-  skip_lws(&s);
+	  /* Parse protocol */
+	  if (!IS_DIGIT(*s))
+		  return -1;
+	  w->w_code = strtoul(s, &s, 10);
+	  skip_lws(&s);
 
-  /* Host (and port) */
-  if (msg_hostport_d(&s, &w->w_host, &w->w_port) == -1)
-    return -1;
-  if (msg_quoted_d(&s, &text) == -1)
-    return -1;
-  if (msg_unquote(text, text) == NULL)
-    return -1;
+	  /* Host (and port) */
+	  if (msg_hostport_d(&s, &w->w_host, &w->w_port) == -1)
+		  return -1;
+	  if (msg_quoted_d(&s, &text) == -1)
+		  return -1;
+	  if (msg_unquote(text, text) == NULL)
+		  return -1;
 
-  w->w_text = text;
+	  w->w_text = text;
+	  
+	  msg_parse_next_field_without_recursion();
+  }
 
-  return msg_parse_next_field(home, h, s, slen);
 }
 
 issize_t msg_warning_e(char b[], isize_t bsiz, msg_header_t const *h, int f)
