@@ -1343,8 +1343,6 @@ tag_typedef_t ntatag_client_rport = BOOLTAG_TYPEDEF(client_rport);
  *          or false (zero or NULL pointer)
  *
  * @par Values
- *    - 3 - add "rport" parameter even if was not present in request
- *          @b and if client User-Agent is "Polycom"
  *    - 2 - add "rport" parameter even if was not present in request
  *    - 1 - use "rport" parameter (default)
  *    - 0 - do not use "rport" parameter
@@ -1399,6 +1397,25 @@ tag_typedef_t ntatag_tcp_rport = BOOLTAG_TYPEDEF(tcp_rport);
  */
 tag_typedef_t ntatag_tls_rport = BOOLTAG_TYPEDEF(tls_rport);
 
+/**@def NTATAG_TLS_ORQ_CONNECT_TIMEOUT(x)
+ *
+ * Connect Timeout for outgoing requests using TLS.
+ *
+ * @par Used with
+ *    nua_create(), nua_set_params(),
+ *    nta_agent_create(), nta_agent_set_params()
+ *
+ * @par Parameter type
+ *    unsigned
+ *
+ * @par Values
+ * - 0 - Disabled
+ * - value - Milliseconds
+ *
+ * @NEW_1_13_2
+ */
+tag_typedef_t ntatag_tls_orq_connect_timeout = UINTTAG_TYPEDEF(tls_orq_connect_timeout);
+
 /**@def NTATAG_PRELOAD(x)
  *
  * Preload by N bytes.
@@ -1472,6 +1489,29 @@ tag_typedef_t ntatag_use_naptr = BOOLTAG_TYPEDEF(naptr);
  */
 tag_typedef_t ntatag_use_srv = BOOLTAG_TYPEDEF(srv);
 
+/**@def NTATAG_SRV_503(x)
+ *
+ * If true, try to use another destination from SRV records on 503 response. RFC3263
+ *
+ * The application can disable NTA from using a new route after 503
+ *
+ * @par Used with
+ *    nua_create(), nua_set_params(), agent_recv_response(),
+ *    nta_agent_create(), nta_agent_set_params()
+ *
+ * @par Parameter type
+ *    boolean: true (non-zero or non-NULL pointer)
+ *          or false (zero or NULL pointer)
+ *
+ * @par Values
+ *    - true - enable new destination on 503
+ *    - false - still use the same destination after timeout
+ *
+ * @sa @RFC3263
+ */
+tag_typedef_t ntatag_srv_503 = BOOLTAG_TYPEDEF(srv_503);
+
+
 /**@def NTATAG_RSEQ(x)
  *
  * @RSeq value for nta_outgoing_prack().
@@ -1914,3 +1954,20 @@ tag_typedef_t ntatag_s_tout_response =    USIZETAG_TYPEDEF(s_tout_response);
 /* Internal */
 tag_typedef_t ntatag_delay_sending = BOOLTAG_TYPEDEF(delay_sending);
 tag_typedef_t ntatag_incomplete = BOOLTAG_TYPEDEF(incomplete);
+
+
+tag_typedef_t ntatag_q_in_completed =    SIZETAG_TYPEDEF(q_in_completed);
+tag_typedef_t ntatag_q_in_final_failed =    SIZETAG_TYPEDEF(q_in_final_failed);
+tag_typedef_t ntatag_q_in_inv_completed =    SIZETAG_TYPEDEF(q_in_inv_completed);
+tag_typedef_t ntatag_q_in_inv_confirmed =    SIZETAG_TYPEDEF(q_in_inv_confirmed);
+tag_typedef_t ntatag_q_in_preliminary =    SIZETAG_TYPEDEF(q_in_preliminary);
+tag_typedef_t ntatag_q_in_proceeding =    SIZETAG_TYPEDEF(q_in_proceeding);
+tag_typedef_t ntatag_q_in_terminated =    SIZETAG_TYPEDEF(q_in_terminated);
+
+tag_typedef_t ntatag_q_out_completed =    SIZETAG_TYPEDEF(q_out_completed);
+tag_typedef_t ntatag_q_out_delayed =    SIZETAG_TYPEDEF(q_out_delayed);
+tag_typedef_t ntatag_q_out_inv_calling =    SIZETAG_TYPEDEF(q_out_inv_calling);
+tag_typedef_t ntatag_q_out_inv_completed =    SIZETAG_TYPEDEF(q_out_inv_completed);
+tag_typedef_t ntatag_q_out_inv_proceeding =    SIZETAG_TYPEDEF(q_out_inv_proceeding);
+tag_typedef_t ntatag_q_out_resolving =    SIZETAG_TYPEDEF(q_out_resolving);
+tag_typedef_t ntatag_q_out_terminated =    SIZETAG_TYPEDEF(q_out_terminated);
