@@ -375,6 +375,12 @@ NTA_DLL extern tag_typedef_t ntatag_tls_rport;
 NTA_DLL extern tag_typedef_t ntatag_tls_rport_ref;
 #define NTATAG_TLS_RPORT_REF(x) ntatag_tls_rport_ref, tag_bool_vr(&(x))
 
+NTA_DLL extern tag_typedef_t ntatag_tls_orq_connect_timeout;
+#define NTATAG_TLS_ORQ_CONNECT_TIMEOUT(x) ntatag_tls_orq_connect_timeout, tag_uint_v((x))
+
+NTA_DLL extern tag_typedef_t ntatag_tls_orq_connect_timeout_ref;
+#define NTATAG_TLS_ORQ_CONNECT_TIMEOUT_REF(x) ntatag_tls_orq_connect_timeout_ref, tag_uint_vr(&(x))
+
 NTA_DLL extern tag_typedef_t ntatag_preload;
 #define NTATAG_PRELOAD(x) ntatag_preload, tag_uint_v((x))
 
@@ -393,6 +399,12 @@ NTA_DLL extern tag_typedef_t ntatag_use_srv;
 NTA_DLL extern tag_typedef_t ntatag_use_srv_ref;
 #define NTATAG_USE_SRV_REF(x) ntatag_use_srv_ref, tag_bool_vr(&(x))
 
+NTA_DLL extern tag_typedef_t ntatag_srv_503;
+#define NTATAG_SRV_503(x) ntatag_srv_503, tag_bool_v((x))
+
+NTA_DLL extern tag_typedef_t ntatag_srv_503_ref;
+#define NTATAG_SRV_503_REF(x) ntatag_srv_503_ref, tag_bool_vr(&(x))
+
 NTA_DLL extern tag_typedef_t ntatag_rseq;
 #define NTATAG_RSEQ(x)    ntatag_rseq, tag_uint_v((x))
 
@@ -594,6 +606,93 @@ NTA_DLL extern tag_typedef_t ntatag_s_tout_response;
 NTA_DLL extern tag_typedef_t ntatag_s_tout_response_ref;
 #define NTATAG_S_TOUT_RESPONSE_REF(x) ntatag_s_tout_response_ref, tag_usize_vr(&(x))
 
+/* ====================================================================== */
+/* Tags for queue status. */
+
+NTA_DLL extern tag_typedef_t ntatag_q_in_completed;
+#define NTATAG_Q_IN_COMPLETED(x) ntatag_q_in_completed, tag_size_v(x)
+
+NTA_DLL extern tag_typedef_t ntatag_q_in_completed_ref;
+#define NTATAG_Q_IN_COMPLETED_REF(x) ntatag_q_in_completed_ref, tag_size_vr(&(x))
+
+NTA_DLL extern tag_typedef_t ntatag_q_in_final_failed;
+#define NTATAG_Q_IN_FINAL_FAILED(x) ntatag_q_in_final_failed, tag_size_v(x)
+
+NTA_DLL extern tag_typedef_t ntatag_q_in_final_failed_ref;
+#define NTATAG_Q_IN_FINAL_FAILED_REF(x) ntatag_q_in_final_failed_ref, tag_size_vr(&(x))
+
+NTA_DLL extern tag_typedef_t ntatag_q_in_inv_completed;
+#define NTATAG_Q_IN_INV_COMPLETED(x) ntatag_q_in_inv_completed, tag_size_v(x)
+
+NTA_DLL extern tag_typedef_t ntatag_q_in_inv_completed_ref;
+#define NTATAG_Q_IN_INV_COMPLETED_REF(x) ntatag_q_in_inv_completed_ref, tag_size_vr(&(x))
+
+NTA_DLL extern tag_typedef_t ntatag_q_in_inv_confirmed;
+#define NTATAG_Q_IN_INV_CONFIRMED(x) ntatag_q_in_inv_confirmed, tag_size_v(x)
+
+NTA_DLL extern tag_typedef_t ntatag_q_in_inv_confirmed_ref;
+#define NTATAG_Q_IN_INV_CONFIRMED_REF(x) ntatag_q_in_inv_confirmed_ref, tag_size_vr(&(x))
+
+NTA_DLL extern tag_typedef_t ntatag_q_in_preliminary;
+#define NTATAG_Q_IN_PRELIMINARY(x) ntatag_q_in_preliminary, tag_size_v(x)
+
+NTA_DLL extern tag_typedef_t ntatag_q_in_preliminary_ref;
+#define NTATAG_Q_IN_PRELIMINARY_REF(x) ntatag_q_in_preliminary_ref, tag_size_vr(&(x))
+
+NTA_DLL extern tag_typedef_t ntatag_q_in_proceeding;
+#define NTATAG_Q_IN_PROCEEDING(x) ntatag_q_in_proceeding, tag_size_v(x)
+
+NTA_DLL extern tag_typedef_t ntatag_q_in_proceeding_ref;
+#define NTATAG_Q_IN_PROCEEDING_REF(x) ntatag_q_in_proceeding_ref, tag_size_vr(&(x))
+
+NTA_DLL extern tag_typedef_t ntatag_q_in_terminated;
+#define NTATAG_Q_IN_TERMINATED(x) ntatag_q_in_terminated, tag_size_v(x)
+
+NTA_DLL extern tag_typedef_t ntatag_q_in_terminated_ref;
+#define NTATAG_Q_IN_TERMINATED_REF(x) ntatag_q_in_terminated_ref, tag_size_vr(&(x))
+
+NTA_DLL extern tag_typedef_t ntatag_q_out_completed;
+#define NTATAG_Q_OUT_COMPLETED(x) ntatag_q_out_completed, tag_size_v(x)
+
+NTA_DLL extern tag_typedef_t ntatag_q_out_completed_ref;
+#define NTATAG_Q_OUT_COMPLETED_REF(x) ntatag_q_out_completed_ref, tag_size_vr(&(x))
+
+NTA_DLL extern tag_typedef_t ntatag_q_out_delayed;
+#define NTATAG_Q_OUT_DELAYED(x) ntatag_q_out_delayed, tag_size_v(x)
+
+NTA_DLL extern tag_typedef_t ntatag_q_out_delayed_ref;
+#define NTATAG_Q_OUT_DELAYED_REF(x) ntatag_q_out_delayed_ref, tag_size_vr(&(x))
+
+NTA_DLL extern tag_typedef_t ntatag_q_out_inv_calling;
+#define NTATAG_Q_OUT_INV_CALLING(x) ntatag_q_out_inv_calling, tag_size_v(x)
+
+NTA_DLL extern tag_typedef_t ntatag_q_out_inv_calling_ref;
+#define NTATAG_Q_OUT_INV_CALLING_REF(x) ntatag_q_out_inv_calling_ref, tag_size_vr(&(x))
+
+NTA_DLL extern tag_typedef_t ntatag_q_out_inv_completed;
+#define NTATAG_Q_OUT_INV_COMPLETED(x) ntatag_q_out_inv_completed, tag_size_v(x)
+
+NTA_DLL extern tag_typedef_t ntatag_q_out_inv_completed_ref;
+#define NTATAG_Q_OUT_INV_COMPLETED_REF(x) ntatag_q_out_inv_completed_ref, tag_size_vr(&(x))
+
+NTA_DLL extern tag_typedef_t ntatag_q_out_inv_proceeding;
+#define NTATAG_Q_OUT_INV_PROCEEDING(x) ntatag_q_out_inv_proceeding, tag_size_v(x)
+
+NTA_DLL extern tag_typedef_t ntatag_q_out_inv_proceeding_ref;
+#define NTATAG_Q_OUT_INV_PROCEEDING_REF(x) ntatag_q_out_inv_proceeding_ref, tag_size_vr(&(x))
+
+NTA_DLL extern tag_typedef_t ntatag_q_out_resolving;
+#define NTATAG_Q_OUT_RESOLVING(x) ntatag_q_out_resolving, tag_size_v(x)
+
+NTA_DLL extern tag_typedef_t ntatag_q_out_resolving_ref;
+#define NTATAG_Q_OUT_RESOLVING_REF(x) ntatag_q_out_resolving_ref, tag_size_vr(&(x))
+
+NTA_DLL extern tag_typedef_t ntatag_q_out_terminated;
+#define NTATAG_Q_OUT_TERMINATED(x) ntatag_q_out_terminated, tag_size_v(x)
+
+NTA_DLL extern tag_typedef_t ntatag_q_out_terminated_ref;
+#define NTATAG_Q_OUT_TERMINATED_REF(x) ntatag_q_out_terminated_ref, tag_size_vr(&(x))
+
 SOFIA_END_DECLS
 
 #endif /* !defined(nta_tag_h) */
