@@ -89,7 +89,7 @@ su_log_t nua_log[] = { SU_LOG_INIT("nua", "NUA_DEBUG", SU_DEBUG) };
  * @param root            Pointer to a root object
  * @param callback        Pointer to event callback function
  * @param magic           Pointer to callback context
- * @param tag,value,... List of tagged parameters
+ * @param tag,value, ... List of tagged parameters
  *
  * @retval !=NULL a pointer to a @nua stack object
  * @retval NULL upon an error
@@ -99,14 +99,14 @@ su_log_t nua_log[] = { SU_LOG_INIT("nua", "NUA_DEBUG", SU_DEBUG) };
  *   (but see also NUTAG_INITIAL_ROUTE()).
  * - NUTAG_URL() (and NUTAG_SIPS_URL(), listing URIs describing
  *   transports)
- * - NUTAG_CERTIFICATE_DIR(), specifying the location of the 
+ * - NUTAG_CERTIFICATE_DIR(), specifying the location of the
  *   root and client/server certificate files
- * - NUTAG_SIP_PARSER(), providing customized parser used to 
+ * - NUTAG_SIP_PARSER(), providing customized parser used to
  *   parse received SIP messages
  * - All parameter tags, listed with nua_set_params()
  * - All NTATAG_* are passed to NTA documented in <sofia-sip/nta_tag.h>:
- *   see NTATAG_EXTRA_100(), 
- * - All tport tags are passed to tport. 
+ *   see NTATAG_EXTRA_100(),
+ * - All tport tags are passed to tport.
  *   They are documented in <sofia-sip/tport_tag.h>
  * - All SOATAG_* are passed to the default SOA (media session) object which
  *   is created by nua_create() unless NUTAG_MEDIA_ENABLE(0) is included in
@@ -215,6 +215,7 @@ void nua_shutdown(nua_t *nua)
  */
 void nua_destroy(nua_t *nua)
 {
+  nua_handle_t *nh, *nh_next;
   enter;
 
   if (nua) {
@@ -234,10 +235,32 @@ void nua_destroy(nua_t *nua)
 #if HAVE_SMIME		/* Start NRC Boston */
     sm_destroy(nua->sm);
 #endif			/* End NRC Boston */
-    su_home_unref(nua->nua_home);
+
+    /* Cleanup remaining nua handles as they are su_home_new'ed and not su_home_cloned (do not belong to the nua's home)
+       See nh_create_handle().
+    */
+    for (nh = nua->nua_handles; nh; nh = nh_next) {
+      su_home_t *nh_home = (su_home_t *)nh;
+      nh_next = nh->nh_next;
+
+      /* at least one handle will be found here and it is nua default handle
+         which is su_home_cloned (see nua_stack_init()) and therefore does not actually require to be unrefed
+         but it is safe to do that anyways just for sure
+      */
+      SU_DEBUG_9(("nua(%p): found handle with refcount = "MOD_ZU". Destroying.\n", (void *)nh, su_home_refcount(nh_home)));
+
+      /* nua is about to die so we don't remove nh from nua, just unref nh */
+      while(!su_home_unref(nh_home));
+    }
+
+	nua_unref(nua);
   }
 }
 
+void nua_unref(nua_t *nua) {
+	if (nua) su_home_unref(nua->nua_home);
+}
+
 /** Fetch callback context from nua.
  *
  * @param nua         Pointer to @nua stack object
@@ -1001,6 +1024,12 @@ static int nua_stack_handle_by_replaces_call(void *arg)
   struct nua_stack_handle_by_replaces_args *a = arg;
 
   a->retval = nua_stack_handle_by_replaces(a->nua, a->r);
+  if (!NH_IS_VALID(a->retval) || NH_IS_DEFAULT(a->retval)) {
+    a->retval = NULL;
+    return 1;
+  }
+
+  nua_handle_ref(a->retval);
 
   return 0;
 }
@@ -1016,6 +1045,12 @@ static int nua_stack_handle_by_call_id_call(void *arg)
   struct nua_stack_handle_by_call_id_args *a = arg;
 
   a->retval = nua_stack_handle_by_call_id(a->nua, a->call_id);
+  if (!NH_IS_VALID(a->retval) || NH_IS_DEFAULT(a->retval)) {
+    a->retval = NULL;
+    return 1;
+  }
+  
+  nua_handle_ref(a->retval);
 
   return 0;
 }
@@ -1046,10 +1081,7 @@ nua_handle_t *nua_handle_by_replaces(nua_t *nua, sip_replaces_t const *r)
     if (su_task_execute(nua->nua_server,
 			nua_stack_handle_by_replaces_call, (void *)&a,
 			NULL) == 0) {
-      nua_handle_t *nh = a.retval;
-
-      if (nh && !NH_IS_DEFAULT(nh) && nh->nh_valid)
-	return nua_handle_ref(nh);
+       return a.retval;
     }
   }
   return NULL;
@@ -1081,11 +1113,78 @@ nua_handle_t *nua_handle_by_call_id(nua_t *nua, const char *call_id)
     if (su_task_execute(nua->nua_server,
 			nua_stack_handle_by_call_id_call, (void *)&a,
 			NULL) == 0) {
-      nua_handle_t *nh = a.retval;
-
-      if (nh && !NH_IS_DEFAULT(nh) && nh->nh_valid)
-	return nua_handle_ref(nh);
+       return a.retval;
     }
   }
   return NULL;
 }
+
+/** Get leg from dialog. */
+const nta_leg_t *nua_get_dialog_state_leg(nua_handle_t *nh)
+{
+	if (nh && nh->nh_ds)
+		return nh->nh_ds->ds_leg;
+	else
+		return NULL;
+}
+
+/** Get su_home_t from nua handle. */
+su_home_t *nua_handle_get_home(nua_handle_t *nh)
+{
+	if (nh && nh->nh_home)
+		return nh->nh_home;
+	else
+		return NULL;
+}
+
+/** Get su_home_t from nua. */
+su_home_t *nua_get_home(nua_t *nua)
+{
+	if (nua && nua->nua_home)
+		return nua->nua_home;
+	else
+		return NULL;
+}
+
+/** Get nta_agent_t from nua. */
+nta_agent_t *nua_get_agent(nua_t *nua)
+{
+	if (nua && nua->nua_nta)
+		return nua->nua_nta;
+	else
+		return NULL;
+}
+
+/** Set has invite of a nua handle */
+void nua_handle_set_has_invite(nua_handle_t *nh, unsigned val)
+{
+	if (nh)
+		nh->nh_has_invite = val;
+}
+
+/** Check if nua handle is destroyed */
+unsigned nua_handle_is_destroyed(nua_handle_t *nh)
+{
+	assert(nh);
+	return nh->nh_destroyed;
+}
+
+void nua_handle_dialog_usage_set_refresh_range(nua_handle_t *nh,
+	unsigned min, unsigned max) {
+	if (nh && nh->nh_ds && nh->nh_ds->ds_usage) {
+		nua_dialog_usage_set_refresh_range(nh->nh_ds->ds_usage, min, max);
+	}
+}
+
+void nua_unref_user(nua_t *nua)
+{
+	enter;
+	nua_signal(nua, NULL, NULL, nua_r_unref, 0, NULL, TAG_NULL());
+}
+
+void nua_handle_unref_user(nua_handle_t *nh)
+{
+	assert(nh);
+	nh_enter;
+	nua_signal(nh->nh_nua, nh, NULL, nua_r_handle_unref, 0, NULL, TAG_NULL());
+}
