@@ -122,6 +122,7 @@ int nua_stack_process_request(nua_handle_t *nh,
 
   switch (sip->sip_request->rq_url->url_type) {
   case url_sip:
+  case url_urn:
   case url_sips:
   case url_im:
   case url_pres:
@@ -158,14 +159,15 @@ int nua_stack_process_request(nua_handle_t *nh,
     /* These must be in-dialog */
     sm = NULL;
   }
-  else if (initial && sip->sip_to->a_tag) {
+  else if (initial && sip->sip_to->a_tag && method != sip_method_subscribe) {
     /* RFC 3261 section 12.2.2:
 
        If the UAS wishes to reject the request because it does not wish to
        recreate the dialog, it MUST respond to the request with a 481
        (Call/Transaction Does Not Exist) status code and pass that to the
        server transaction.
-    */
+    */ /* we allow this on subscribes because we have disabled the built-in notify server and we need those messages in the application layer */
+	  
     if (method == sip_method_info)
       /* accept out-of-dialog info */; else
     if (method != sip_method_message || !NH_PGET(nh, win_messenger_enable))
@@ -260,11 +262,13 @@ int nua_stack_process_request(nua_handle_t *nh,
   }
 
   if (sr->sr_status <= 100) {
-    SR_STATUS1(sr, SIP_100_TRYING);
-    if (method == sip_method_invite || sip->sip_timestamp) {
-      nta_incoming_treply(irq, SIP_100_TRYING,
-			  SIPTAG_USER_AGENT_STR(user_agent),
-			  TAG_END());
+	  	  SR_STATUS1(sr, SIP_100_TRYING);
+    if ((method == sip_method_invite && nh->nh_prefs->nhp_auto_invite_100) ||
+        sip->sip_timestamp) {
+		nta_incoming_treply(irq, SIP_100_TRYING,
+							SIPTAG_USER_AGENT_STR(user_agent),
+							TAG_END());
+
     }
   }
   else {
@@ -293,31 +297,43 @@ int nua_base_server_preprocess(nua_server_request_t *sr)
 
 void nua_server_request_destroy(nua_server_request_t *sr)
 {
+  nua_server_request_t *sr0 = NULL;
+
   if (sr == NULL)
     return;
 
   if (SR_HAS_SAVED_SIGNAL(sr))
     nua_destroy_signal(sr->sr_signal);
 
-  if (sr->sr_irq) {
-    if (sr->sr_method == sip_method_bye && sr->sr_status < 200) {
-      nta_incoming_treply(sr->sr_irq, SIP_200_OK, TAG_END());
-    }
-    nta_incoming_destroy(sr->sr_irq), sr->sr_irq = NULL;
-  }
-
-  if (sr->sr_request.msg)
-    msg_destroy(sr->sr_request.msg), sr->sr_request.msg = NULL;
-
-  if (sr->sr_response.msg)
-    msg_destroy(sr->sr_response.msg), sr->sr_response.msg = NULL;
-
   if (sr->sr_prev) {
     /* Allocated from heap */
     if ((*sr->sr_prev = sr->sr_next))
       sr->sr_next->sr_prev = sr->sr_prev;
-    su_free(sr->sr_owner->nh_home, sr);
+	sr0 = sr;
   }
+
+  if (sr->sr_irq) {
+	nta_incoming_t *irq = sr->sr_irq;
+    if (sr->sr_method == sip_method_bye && sr->sr_status < 200) {
+      nta_incoming_treply(sr->sr_irq, SIP_200_OK, TAG_END());
+    }
+	sr->sr_irq = NULL;
+    nta_incoming_destroy(irq);
+  }
+
+  if (sr->sr_request.msg) {
+	msg_t *msg = sr->sr_request.msg;
+	sr->sr_request.msg = NULL;
+    msg_destroy(msg);
+  }
+
+  if (sr->sr_response.msg) {
+	msg_t *msg = sr->sr_response.msg;
+	sr->sr_response.msg = NULL;
+    msg_destroy(msg); 
+  }
+
+  if (sr0) su_free(sr->sr_owner->nh_home, sr0);
 }
 
 /**@fn void nua_respond(nua_handle_t *nh, int status, char const *phrase, tag_type_t tag, tag_value_t value, ...);
@@ -445,7 +461,12 @@ nua_stack_respond(nua_t *nua, nua_handle_t *nh,
 
   nua_server_params(sr, tags);
   nua_server_respond(sr, tags);
-  nua_server_report(sr);
+
+  if (!(sr->sr_method == sip_method_invite && status == 100)) {
+    /* Since we don't change state, do not notify application when
+       we send 100 Trying for INVITE */
+    nua_server_report(sr);
+  }
 }
 
 int nua_server_params(nua_server_request_t *sr, tagi_t const *tags)
@@ -509,10 +530,18 @@ int nua_server_respond(nua_server_request_t *sr, tagi_t const *tags)
 #endif
 
   if (sr->sr_response.msg == NULL) {
-    assert(sr->sr_status == 500);
+	  //assert(sr->sr_status == 500);
+	  SU_DEBUG_0(("sr without msg, sr_status=%u", sr->sr_status));
     goto internal_error;
   }
 
+  if (sr->sr_status == 100) {
+    return nta_incoming_treply(sr->sr_irq, SIP_100_TRYING,
+                               SIPTAG_USER_AGENT_STR(NH_PGET(nh, user_agent)),
+                               TAG_END());
+    return 0;
+  }
+
   if (sr->sr_status < 200) {
     next.msg = nta_incoming_create_response(sr->sr_irq, 0, NULL);
     next.sip = sip_object(next.msg);
@@ -536,6 +565,10 @@ int nua_server_respond(nua_server_request_t *sr, tagi_t const *tags)
 	   sip_add_make(msg, sip, sip_organization_class,
 			NH_PGET(nh, organization)) < 0)
     ;
+  else if (!sip->sip_via && NH_PGET(nh, via) &&
+	   sip_add_make(msg, sip, sip_via_class,
+			NH_PGET(nh, via)) < 0)
+    ;
   else if (!sip->sip_allow && NH_PGET(nh, allow) &&
 	   sip_add_dup(msg, sip, (void *)NH_PGET(nh, allow)) < 0)
     ;
@@ -697,12 +730,11 @@ int nua_base_server_report(nua_server_request_t *sr, tagi_t const *tags)
 
   if (initial && 300 <= status)
     terminated = 1;
+  else if (sr->sr_terminating && status < 300)
+    terminated = 1;
   else
     terminated = sip_response_terminates_dialog(status, sr->sr_method, NULL);
 
-  if (sr->sr_terminating & !terminated)
-    terminated = 1;
-
   if (usage && terminated)
     nua_dialog_usage_remove(nh, nh->nh_ds, usage, NULL, sr);
 
