@@ -376,8 +376,8 @@ void nua_application_event(nua_t *dummy, su_msg_r sumsg, nua_ee_data_t *ee)
       char const *name = nua_event_name((enum nua_event_e)e->e_event) + 4;
       SU_DEBUG_7(("nua(%p): event %s dropped\n", (void *)nh, name));
     }
-    nua_handle_unref(nh);
-    nua_stack_unref(nua);
+    if (nh) nua_handle_unref_user(nh);
+    if (nua) nua_unref_user(nua);
     return;
   }
 
@@ -402,8 +402,8 @@ void nua_application_event(nua_t *dummy, su_msg_r sumsg, nua_ee_data_t *ee)
     nua->nua_current = frame->nf_next;
   }
 
-  nua_handle_unref(nh);
-  nua_stack_unref(nua);
+  if (nh) nua_handle_unref_user(nh);
+  if (nua) nua_unref_user(nua);
 }
 
 /** Get current request message. @NEW_1_12_4.
@@ -492,7 +492,7 @@ int nua_signal(nua_t *nua, nua_handle_t *nh, msg_t *msg,
   if (nua == NULL)
     return -1;
 
-  if (nua->nua_shutdown_started && event != nua_r_shutdown)
+  if (nua->nua_shutdown_started && event != nua_r_shutdown && event != nua_r_destroy && event != nua_r_handle_unref && event != nua_r_unref)
     return -1;
 
   ta_start(ta, tag, value);
@@ -514,7 +514,7 @@ int nua_signal(nua_t *nua, nua_handle_t *nh, msg_t *msg,
 
     assert(tend == t); (void)tend; assert(b == bend); (void)bend;
 
-    e->e_always = event == nua_r_destroy || event == nua_r_shutdown;
+    e->e_always = event == nua_r_destroy || event == nua_r_shutdown || event == nua_r_handle_unref || event == nua_r_unref;
     e->e_event = event;
     e->e_nh = nh ? nua_handle_ref(nh) : NULL;
     e->e_status = status;
@@ -549,10 +549,10 @@ void nua_stack_signal(nua_t *nua, su_msg_r msg, nua_ee_data_t *ee)
   nua_event_data_t *e = ee->ee_data;
   nua_handle_t *nh = e->e_nh;
   tagi_t *tags = e->e_tags;
-  nua_event_t event;
+  nua_event_t event = (enum nua_event_e)e->e_event;
   int error = 0;
 
-  if (nh) {
+  if (nh && event != nua_r_handle_unref) {
     if (!nh->nh_prev)
       nh_append(nua, nh);
     if (!nh->nh_ref_by_stack) {
@@ -575,8 +575,6 @@ void nua_stack_signal(nua_t *nua, su_msg_r msg, nua_ee_data_t *ee)
 
   su_msg_save(nua->nua_signal, msg);
 
-  event = (enum nua_event_e)e->e_event;
-
   if (nua->nua_shutdown && !e->e_always) {
     /* Shutting down */
     nua_stack_event(nua, nh, NULL, event,
