@@ -179,6 +179,14 @@ tag_typedef_t tptag_idle = UINTTAG_TYPEDEF(idle);
  */
 tag_typedef_t tptag_timeout = UINTTAG_TYPEDEF(timeout);
 
+/**@def TPTAG_SOCKET_KEEPALIVE(x)
+ *
+ * Keepalive interval set on socket (where supported) in seconds.
+ *
+ * If 0 or UINT_MAX, do not use keepalives. Default value is 30.
+ */
+tag_typedef_t tptag_socket_keepalive = UINTTAG_TYPEDEF(socket_keepalive);
+
 /**@def TPTAG_KEEPALIVE(x)
  *
  * Keepalive interval in milliseconds.
@@ -270,6 +278,14 @@ tag_typedef_t tptag_certificate = STRTAG_TYPEDEF(certificate);
  */
 tag_typedef_t tptag_compartment = PTRTAG_TYPEDEF(compartment);
 
+/**@def TPTAG_TLS_CIPHERS(x)
+ *
+ * Sets the supported TLS cipher suites.
+ *
+ * Use with tport_tbind(), nua_create(), nta_agent_create(),
+ * nta_agent_add_tport(), nth_engine_create(), or initial nth_site_create().
+ */
+tag_typedef_t tptag_tls_ciphers = STRTAG_TYPEDEF(tls_ciphers);
 
 /**@def TPTAG_TLS_VERSION(x)
  *
@@ -280,6 +296,19 @@ tag_typedef_t tptag_compartment = PTRTAG_TYPEDEF(compartment);
  */
 tag_typedef_t tptag_tls_version = UINTTAG_TYPEDEF(tls_version);
 
+/**@def TPTAG_TLS_TIMEOUT(x)
+ *
+ * Sets the maximum TLS session lifetime in seconds.
+ *
+ * The default value is 300 seconds.
+ *
+ * Use with tport_tbind(), nua_create(), nta_agent_create(),
+ * nta_agent_add_tport(), nth_engine_create(), or initial nth_site_create().
+ *
+ * @NEW_UNRELEASED.
+ */
+tag_typedef_t tptag_tls_timeout = UINTTAG_TYPEDEF(tls_timeout);
+
 /**@def TPTAG_TLS_VERIFY_PEER(x)
  * @par Depreciated:
  *    Alias for TPTAG_TLS_VERIFY_POLICY(TPTLS_VERIFY_IN|TPTLS_VERIFY_OUT)
@@ -302,25 +331,25 @@ tag_typedef_t tptag_tls_passphrase = STRTAG_TYPEDEF(tls_passphrase);
  *
  * The verification of certificates can be controlled:
  * @par Values:
- *    - #TPTLS_VERIFY_NONE:
+ *    - #TPTLS_VERIFY_NONE: 
  *          Do not verify Peer Certificates.
- *    - #TPTLS_VERIFY_IN:
- *          Drop incoming connections which fail signature verification
- *          against trusted certificate authorities. Peers must provide a
+ *    - #TPTLS_VERIFY_IN: 
+ *          Drop incoming connections which fail signature verification 
+ *          against trusted certificate authorities. Peers must provide a 
  *          certificate during the initial TLS Handshake.
- *    - #TPTLS_VERIFY_OUT:
- *          Drop outgoing connections which fail signature verification
+ *    - #TPTLS_VERIFY_OUT: 
+ *          Drop outgoing connections which fail signature verification 
  *          against trusted certificate authorities.
- *    - #TPTLS_VERIFY_ALL:
+ *    - #TPTLS_VERIFY_ALL: 
  *          Alias for (TPTLS_VERIFY_IN|TPTLS_VERIFY_OUT)
- *    - #TPTLS_VERIFY_SUBJECTS_IN:
- *          Match the certificate subject on incoming connections against
- *          a provided list.  If no match is found, the connection is
+ *    - #TPTLS_VERIFY_SUBJECTS_IN: 
+ *          Match the certificate subject on incoming connections against 
+ *          a provided list.  If no match is found, the connection is 
  *          rejected. If no list is provided, subject checking is bypassed.
  *          Note: Implies #TPTLS_VERIFY_IN.
- *    - #TPTLS_VERIFY_SUBJECTS_OUT:
- *          Match the certificate subject on outgoing connections against
- *          a provided list.  If no match is found, the connection is
+ *    - #TPTLS_VERIFY_SUBJECTS_OUT: 
+ *          Match the certificate subject on outgoing connections against 
+ *          a provided list.  If no match is found, the connection is 
  *          rejected.
  *          Note: Implies #TPTLS_VERIFY_OUT.
  *    - #TPTLS_VERIFY_SUBJECTS_ALL:
@@ -338,12 +367,12 @@ tag_typedef_t tptag_tls_verify_policy = UINTTAG_TYPEDEF(tls_verify_policy);
 /**@def TPTAG_TLS_VERIFY_DEPTH(x)
  *
  * Define the maximum length of a valid certificate chain.
- *
+ * 
  * @par Default
  *   2
  *
  * @par Used with
- *   tport_tbind(), nua_create(), nta_agent_create(), nta_agent_add_tport(),
+ *   tport_tbind(), nua_create(), nta_agent_create(), nta_agent_add_tport(), 
  *   nth_engine_create(), or initial nth_site_create().
  *
  * @par Parameter Type:
@@ -366,7 +395,7 @@ tag_typedef_t tptag_tls_verify_depth = UINTTAG_TYPEDEF(tls_verify_depth);
  *   - Non-Zero - Enable date verification.
  *
  * @par Used with
- *   tport_tbind(), nua_create(), nta_agent_create(), nta_agent_add_tport(),
+ *   tport_tbind(), nua_create(), nta_agent_create(), nta_agent_add_tport(), 
  *   nth_engine_create(), or initial nth_site_create().
  *
  * @par Parameter Type:
@@ -387,7 +416,7 @@ tag_typedef_t tptag_tls_verify_date = UINTTAG_TYPEDEF(tls_verify_date);
  * the connection is automatically rejected.
  *
  * @par Used with
- *   tport_tbind(), nua_create(), nta_agent_create(), nta_agent_add_tport(),
+ *   tport_tbind(), nua_create(), nta_agent_create(), nta_agent_add_tport(), 
  *   nth_engine_create(), initial nth_site_create(),
  *   TPTLS_VERIFY_SUBJECTS_IN
  *
@@ -407,7 +436,7 @@ tag_typedef_t tptag_tls_verify_subjects = PTRTAG_TYPEDEF(tls_verify_subjects);
 /**@def TPTAG_X509_SUBJECT(x)
  *
  * Requires that a message be sent over a TLS transport with trusted X.509
- * certificate.  The character string provided must match against a subject
+ * certificate.  The character string provided must match against a subject 
  * from the trusted certificate.
  *
  * @par Used with
@@ -564,6 +593,19 @@ tag_typedef_t tptag_log = INTTAG_TYPEDEF(log);
  */
 tag_typedef_t tptag_dump = STRTAG_TYPEDEF(dump);
 
+/**@def TPTAG_CAPT(x)
+ *
+ * URL for capturing unparsed messages from transport.
+ *
+ * Use with tport_tcreate(), nta_agent_create(), nua_create(),
+ * nth_engine_create(), or initial nth_site_create().
+ *
+ * @sa #TPORT_CAPT environment variable, TPTAG_LOG().
+ *
+ */
+tag_typedef_t tptag_capt = STRTAG_TYPEDEF(capt);
+
+
 /** Mark transport as trusted.
  *
  * @note Not implemented by tport module.
